@@ -108,8 +108,8 @@ static void    ltable_remove_if  (LTable* ltable,
 static void    ltable_spew(LTable* ltable);
 #endif
 
-static LTableEntry* ltable_entry_new(const gchar* name,
-                                     const gchar* full_name);
+static LTableEntry* ltable_entry_new(gchar **pathv,
+				     int     end);
 static void         ltable_entry_destroy(LTableEntry* entry);
 
 static Listener* listener_new   (guint      cnxn_id,
@@ -340,7 +340,7 @@ ltable_insert(LTable* lt, const gchar* where, Listener* l)
   
   if (lt->tree == NULL)
     {
-      lte = ltable_entry_new("/", "/");
+      lte = ltable_entry_new(NULL, 0);
       
       lt->tree = g_node_new(lte);
 
@@ -389,7 +389,7 @@ ltable_insert(LTable* lt, const gchar* where, Listener* l)
 
       if (found == NULL)
         {
-          ne = ltable_entry_new(dirnames[i], where);
+          ne = ltable_entry_new(dirnames, i);
               
           if (across != NULL) /* Across is at the one past */
             found = g_node_insert_data_before(cur, across, ne);
@@ -419,8 +419,8 @@ ltable_insert(LTable* lt, const gchar* where, Listener* l)
   lt->active_listeners += 1;
 
 #ifdef DEBUG_LISTENERS
-  printf("Added %u at %s, spewing:\n",
-         l->cnxn, where);
+  g_print ("Added %u at %s, spewing:\n",
+	   l->cnxn, where);
   ltable_spew(lt);
 #endif
 }
@@ -523,7 +523,7 @@ ltable_remove(LTable* lt, guint cnxn)
   lt->active_listeners -= 1;
 
 #ifdef DEBUG_LISTENERS
-  printf("Removed %u, spewing:\n", cnxn);
+  g_print ("Removed %u, spewing:\n", cnxn);
   ltable_spew(lt);
 #endif
 }
@@ -826,15 +826,36 @@ ltable_remove_if  (LTable                 *ltable,
 }
 
 static LTableEntry* 
-ltable_entry_new(const gchar* name,
-                 const gchar *full_name)
+ltable_entry_new(gchar **pathv,
+		 int     end)
 {
   LTableEntry* lte;
 
   lte = g_new0(LTableEntry, 1);
 
-  lte->name = g_strdup(name);
-  lte->full_name = g_strdup (full_name);
+  if (pathv != NULL)
+    {
+      GString* full_name;
+      guint i;
+
+      lte->name = g_strdup(pathv[end]);
+
+      full_name = g_string_new("/");
+      i = 0;
+      while (i <= end)
+	{
+	  g_string_append(full_name, pathv[i]);
+	  if (i != end)
+	    g_string_append_c(full_name, '/');
+	  i++;
+	}
+      lte->full_name = g_string_free(full_name, FALSE);
+    }
+  else
+    {
+      lte->name = g_strdup("/");
+      lte->full_name = g_strdup("/");
+    }
   
   return lte;
 }
@@ -860,20 +881,20 @@ spew_func(GNode* node, gpointer data)
   spaces[g_node_depth(node)] = '\0';
 
   
-  printf(" %sSpewing node `%s' (%p): ", spaces, lte->name, node);
+  g_print (" %sSpewing node `%s' (%p): ", spaces, lte->name, node);
 
   tmp = lte->listeners;
   while (tmp != NULL)
     {
       Listener* l = tmp->data;
       
-      printf("  %slistener %u is here\n", spaces, (guint)l->cnxn);
+      g_print ("  %slistener %u is here\n", spaces, (guint)l->cnxn);
 
       tmp = g_list_next(tmp);
     }
 
   if (lte->listeners == NULL)
-    printf("\n");
+    g_print ("\n");
   
   return FALSE;
 }
@@ -882,13 +903,13 @@ static void
 ltable_spew(LTable* lt)
 {
   guint i;
-  printf("Flat table:\n");
+  g_print ("Flat table:\n");
   i = 0;
   while (i < lt->listeners->len)
     {
       GNode* node = g_ptr_array_index(lt->listeners, i);
       LTableEntry* lte = node ? node->data : NULL;
-      printf("%u `%s' %p\n", i, lte ? lte->name : "", node);
+      g_print ("%u `%s' %p\n", i, lte ? lte->name : "", node);
       
       ++i;
     }
